package httpdata

import (
	"errors"

	"git.blauwelle.com/go/crate/runtimehelper"
)

const maximumFrames = 8

type UniverseError struct {
	Code      Code
	Message   string
	Debug     string
	Traceback []runtimehelper.Frame
}

func (err UniverseError) Error() string {
	if err.Message == "" {
		return string(err.Code)
	}
	return string(err.Code) + ": " + err.Message
}

func NewUniverseError(code Code, message string) error {
	return UniverseError{
		Code:    code,
		Message: message,
	}
}

func NewBadRequestError(message string) error {
	return UniverseError{
		Code:    CodeBadRequest,
		Message: message,
	}
}

func IsUniverseError(err error) bool {
	return errors.As(err, &UniverseError{})
}

func ToUniverseError(err error, code Code, message string) error {
	if err == nil || IsUniverseError(err) {
		return err
	}
	return NewUniverseError(code, message)
}
